/************************************************************************/
/*    FEDIT 	A tool to view and edit files in hex and ascii.		*/
/*									*/
/* 		Author: W. A. Smethurst					*/
/*		Date:	07/06/95					*/
/************************************************************************/

#include "conio.h"
#include "stdio.h"
#include "stdlib.h"
#include "io.h"
#include "dos.h"
#include "dir.h"
#include "alloc.h"
#include "ctype.h"
#include "errno.h"
#include "time.h"
#include "fcntl.h"
#include "fedit.h"

/* Screen Constants	*/

#define FirstRow	4
#define LastRow		19
#define CmdRow		23
#define MsgRow		24

/************************************************************************/
/* Global Variable Declarations						*/
/************************************************************************/

char *ProgTitle = "FEDIT v1.0 - View/Edit Files in HEX and ASCII";

int FileHandle;			/* File handle for input file		*/
int CurrentPos;			/* Current position in Input Buffer	*/
int ByteCount;			/* Actual count of bytes read		*/
int RowCount;			/* Actual No. of Rows to display	*/
int RecordSize = 256;

long MaxRecords;		/* Maximum records on input file	*/
long FileLength;		/* File Length				*/
long NextRelRecNo;		/* Next data record relative record no.	*/
long ByteOffset;		/* Byte Offset into File		*/

unsigned FileDate,FileTime;

BOOL DataChanged;		/* TRUE if data was changed		*/

/************************************************************************/
/* Global Buffer, Array and String Storage Declarations			*/
/************************************************************************/

char MsgBuffer[80], InputFile[MAXPATH], SearchString[33], HexBuffer[16];

char *InputBuffer;		/* Record buffer			*/

/************************************************************************/
/* External data declarations						*/
/************************************************************************/

extern char *sys_errlist[];

/************************************************************************/
/* FEDIT Main Program Function						*/
/************************************************************************/

main(int argc, char *argv[])
{
	int Instructions(),Initialization(),Mainline();

	int rc;

	if (argc > 1)	/* If an input file was specified, get the name	*/
		{
		if ( (argv[1][0] == '?') )
			{
			Instructions();		  /* Tell it like it is	*/
			exit (0);
			}
		else
			strcpy(InputFile,argv[1]);
		}

	for (rc = Initialization(); rc == SUCCESS; rc = Mainline());

	if ( (rc == FAIL) )
		abort();

	if ( (DataChanged) )
		if ( (WriteRecord()) == FAIL )
			abort();

	if ( (FileHandle != 0) )
		close(FileHandle);

	exit(0);		/* Exit back to Operating System	*/
}

/************************************************************************/
/* Program Initialization routine					*/
/************************************************************************/

int Initialization()
{
	int rc,srow;
	char *f;

/* Make sure the file exists before tinkering with the display	*/

	if ( (rc = OpenFile()) != SUCCESS )
		return (rc);	  /* Bad file name or program cancelled	*/

/* Now clear the display screen and build the FEDIT display layout	*/
	ClearScreen();

	Cprintf(0,13,LightCyan,"%s",ProgTitle);

	DrawBox(FirstRow - 1,0,LastRow+1,79,LightRed);

	if ( (f = strrchr(InputFile,BKSLASH)) == NULL )    /* File name	*/
		f = InputFile;
	else
		f++;

	srow = FirstRow - 2;

/* File Information	*/

	Cprintf(srow,2,Yellow,"%s",f);
	Cprintf(srow,16,Yellow,"%s",FileDate);
	Cprintf(srow,26,Yellow,"%s",FileTime);
	Cprintf(srow,46,StdAttr,"Pages:          Page No:");
	Cprintf(srow,34,Yellow,"% 9lu",FileLength);
	Cprintf(srow,53,Yellow,"% 7lu",MaxRecords);
	Cprintf(srow,71,Yellow,"% 7lu",NextRelRecNo + 1);

	srow = LastRow + 2;

/* Navigation Keys	*/

	Cprintf(srow,1,StdAttr,"Scroll: ");
	WriteString("Home End PgUp PgDn",LightCyan);
	Cprintf(srow+1,11,LightCyan,"Tab  Shift-Tab");

	Cprintf(srow,30,StdAttr,"Cursor: ");
	WriteString("\x18\x19\x1A\x1B",LightCyan);
	Cprintf(srow+1,30,LightCyan,"^Right ^Left");

/* Function Keys	*/

	Cprintf(srow,45,StdAttr,"F1: ");
	WriteString("Search in Hex",LightCyan);

	Cprintf(srow+1,45,StdAttr,"F2: ");
	WriteString("Search in ASCII",LightCyan);

	Cprintf(srow+2,45,StdAttr,"F3: ");
	WriteString("Repeat Search",LightCyan);

	Cprintf(srow,67,StdAttr,"F4: ");
	WriteString("Go To Pg",LightCyan);

	Cprintf(srow+1,67,StdAttr,"F5: ");
	WriteString("Update",LightCyan);

/* How to Terminate FEDIT	*/

	Cprintf(srow+2,67,StdAttr,"Escape: ");
	WriteString("Exit",LightCyan);

/* Show the very first record	*/

	if ( (DisplayRecord(0L)) != SUCCESS )
		return (FAIL);

	return (SUCCESS);
}

int Instructions()
{
	printf("\nEnter FEDIT all by itself at the DOS prompt and it will prompt you for a");
	printf("\nfile or path name.  You must provide a valid file or path name for FEDIT");
	printf("\nto work with or it will terminate.\n");

	printf("\nOptionally, you can provide a file or path name on the command line when");
	printf("\nFEDIT is executed by entering: FEDIT pathname.\n");

	return (SUCCESS);
}

/************************************************************************/
/* Program Mainline processing						*/
/************************************************************************/

int Mainline()
{
	int DispatchFunction(unsigned fnc);

	int rc;
	unsigned c;

	ShowChar(CurrentPos,ReverseVideo);  /* Hilite current character	*/

	do {;} while ( (c = InKey()) == NO_KEY ); /* Wait for keystroke	*/

	ClearMessage();			    /* Clear any prior messages	*/

	ShowChar(CurrentPos,StdAttr);	  /* Turn off current character	*/

	rc = DispatchFunction(c);		/* Handle the keystroke	*/

	return (rc);
}

int DispatchFunction(unsigned fnc)
{
	long GetPageNumber();
	int GetHex(),GetAscii(),ChangeData(unsigned ch),RepeatSearch();

	long tnrno;

	switch (fnc)
		{
		case CANCEL:			/* Stick a fork in me	*/
			SetCursor(24,0);	/* I'm done...	*/
			return (EXIT);
		case BS:
		case LEFT:
			if ( (CurrentPos > 0) )
				CurrentPos--;

			break;
		case RIGHT:
			if ( (CurrentPos < (ByteCount - 1)) )
				CurrentPos++;

			break;
		case UP:
			if ( ((CurrentPos / 16) > 0) )
				{
				CurrentPos -= 16;
				break;
				}

			if ( (NextRelRecNo > 0L) )
				{
				if ( (DisplayRecord(NextRelRecNo - 1)) == FAIL )
					return (FAIL);

				CurrentPos = ((RowCount - 1) * 16) + (CurrentPos % 16);
				}

			break;
		case DOWN:
			if ( (CurrentPos / 16 < (RowCount - 1)) )
				{
				CurrentPos += 16;
				break;
				}

			if ( (NextRelRecNo < (MaxRecords - 1)) )
				{
				if ( (DisplayRecord(NextRelRecNo + 1)) == FAIL )
					return (FAIL);

				CurrentPos = CurrentPos % 16;
				}

			break;
		case CTL_HOME:	     /* First character of current page	*/
			CurrentPos = 0;
			break;
		case CTL_END:	      /* Last character of current page	*/
			CurrentPos = ByteCount - 1;
			break;
		case SHFT_TAB:			  /* Leftmost character	*/
		case CTL_LEFT:
			CurrentPos = (CurrentPos / 16) * 16;
			break;
		case TAB:			 /* Rightmost character	*/
		case CTL_RIGHT:
			CurrentPos = ((CurrentPos / 16) * 16) + 15;
			break;
		case CTL_PGUP:			 /* Top of current page	*/
			CurrentPos = CurrentPos % 16;
			break;
		case CTL_PGDN:		      /* Bottom of current page	*/
			CurrentPos = ((RowCount - 1) * 16) + (CurrentPos % 16);
			break;
		case HOME:		     /* First character in file	*/
			if ( (DisplayRecord(0L)) == FAIL )
				return (FAIL);

			CurrentPos = 0;
			break;
		case END:		      /* Last character in file	*/
			if ( (DisplayRecord(MaxRecords - 1)) == FAIL )
				return (FAIL);

			CurrentPos = ByteCount - 1;
			break;
		case PGUP:	  /* First row of page or previous page	*/
			if ( ((CurrentPos / 16) > 0) )
				{
				CurrentPos = CurrentPos % 16;
				break;
				}

			if ( (NextRelRecNo > 0L) )
				if ( (DisplayRecord(NextRelRecNo - 1)) == FAIL )
					return (FAIL);

			break;
		case PGDN:	       /* Last row of page or next page	*/
			if ( (CurrentPos / 16) < (RowCount - 1) )
				{
				CurrentPos = ((RowCount - 1) * 16) + (CurrentPos % 16);
				break;
				}				

			if ( (NextRelRecNo < (MaxRecords - 1)) )
				if ( (DisplayRecord(NextRelRecNo + 1)) == FAIL )
					return (FAIL);

			break;

/* Put the Functions at the end because they take more time	*/

		case F1:		       /* Search for Hex string	*/
			return (GetHex());
		case F2:		     /* Search for ASCII string	*/
			return (GetAscii());
		case F3:		       /* Go to a specific page	*/
			if ( (SearchString[0] != NULL) )
				return (RepeatSearch());

			break;
		case F4:
			if ( (tnrno = GetPageNumber()) != -1L )
				if ( (DisplayRecord(tnrno)) == FAIL )
					return (FAIL);

			break;
		case F5:
			if ( (DataChanged) )
				if ( (WriteRecord()) == FAIL )
					return (FAIL);

			break;
		default:			      /* Enter Hex data	*/
			if ( (isxdigit(fnc)) )
				if ( (ChangeData(fnc)) == FAIL )
					return (FAIL);

			break;
		}

	return (SUCCESS);
}

int ChangeData(unsigned ch)
{
	unsigned c,h,hex,row,col;

	ShowChar(CurrentPos,ReverseVideo);	    /* Hilite Character	*/

	c = *(InputBuffer + CurrentPos);     /* Save original character	*/

	row = Row();
	col = Col();

	WriteCharacter(hex = toupper(ch),Yellow);   /* Show left nibble	*/

	SetCursor(row,++col);			  /* Advance the cursor	*/

	h = Convert2Hex(hex) << 4;	  /* Save nibble left justified	*/

	do {	hex = InKey();			/* Get rightmost nibble	*/

		if ( (hex == CANCEL) )
			{
			WriteCharacter(c,StdAttr);  /* Restore original	*/
			return (SUCCESS);
			}

		} while ( !(isxdigit(hex)) );  /* Must be hex character	*/

	WriteCharacter(hex = toupper(hex),Yellow); /* Show right nibble	*/

	h = h | Convert2Hex(hex);     /* Combine left and right nibbles	*/

	if ( (c != h) )				    /* Has it changed ?	*/
		{
		*(InputBuffer + CurrentPos) = h;	/* Yes	*/
		ShowChar(CurrentPos,Yellow);
		DataChanged = TRUE;		/* Turn on changed flag	*/
		}
	else
		ShowChar(CurrentPos,StdAttr);    /* Re-display original	*/

	CurrentPos++;			/* Advance to the next position	*/

	return (SUCCESS);
}

long GetPageNumber()
{
	int c;
	char *rstring;
	long rrn;

	for (EVER)
		{
		rstring = GetString("Page Number ? ",7);

		ClearMessage();

		if ( (*(rstring) == NULL) || (*(rstring) == CANCEL) )
			{
			ShowMessage("Go To Cancelled...");
			return (-1L);		   /* Don't do anything	*/
			}

		rrn = strtol(rstring,NULL,0);   /* Make it long integer	*/

		if ( (rrn > 0L) && (rrn <= MaxRecords) ) /* Range Check	*/
			break;

		ShowMessage("Page number %lu is out of range",rrn);
		}

	return (--rrn);
}

int GetAscii()
{	
	int FindString(char *cbuff,unsigned strlth);

	int i,c,lth;
	char *rstring;

	SearchString[0] = HexBuffer[0] = NULL;

	if ( (DataChanged) )	      /* If anything changed, update it	*/
		if ( (WriteRecord()) == FAIL )
			abort();

	ShowChar(CurrentPos,ReverseVideo);	    /* Hilite Character	*/

	rstring = GetString("Search for ? ",15);  /* Get up to 15 chars	*/

	if ( (*(rstring) == NULL) || (*(rstring) == CANCEL) )
		{
		ShowMessage("Search Cancelled...");
		return (SUCCESS);
		}

	strcpy(SearchString,rstring);     /* Search is case insensitive	*/
	strupr(SearchString);	    /* But let's be neat and consistent	*/

	lth = strlen(SearchString);

	for ( i = 0; i < lth; i++ )	    /* Check for non-printables	*/
		if ( !(isprint(SearchString[i])) )
			SearchString[i] = '.';

	FindString(SearchString,lth);	  /* Search forward in the file	*/

	return (SUCCESS);
}

int GetHex()
{	
	int FindString(char *cbuff,unsigned strlth);

	int i,x,lth;
	unsigned c,h;
	char *rstring;

	SearchString[0] = HexBuffer[0] = NULL;

	if ( (DataChanged) )	      /* If anything changed, update it	*/
		if ( (WriteRecord()) == FAIL )
			abort();

	ShowChar(CurrentPos,ReverseVideo);	    /* Hilite Character	*/

	rstring = GetString("Search for ? ",30);  /* Get up to 30 chars	*/

	if ( (*(rstring) == NULL) || (*(rstring) == CANCEL) )
		{
		ShowMessage("Search Cancelled...");
		return (SUCCESS);
		}

	strcpy(SearchString,rstring);
	strupr(SearchString);		       /* Hex must be upshifted	*/

	lth = strlen(SearchString);

	for (i = x = 0; i < lth; i++)		  /* Convert to raw hex	*/
		{
		c = SearchString[i];

		if ( (isxdigit(c)) == 0 )
			{
			c = '0';	       /* Invalid Hex character	*/
			SearchString[i] = '.';
			}

		c = Convert2Hex(c);	/* Strip the zone and force A-F	*/

		if ( (i & 0x01) == 0 )			/* Odd or Even?	*/
			{
			h = c << 4;		     /* Odd, shift left	*/
			HexBuffer[x] = h;	   /* Save just in case	*/
			}
		else
			{
			h = h | c;		     /* Even, insert it	*/
			HexBuffer[x++] = h;		   /* and store	*/
			}
		}

	HexBuffer[x] = 0;		  /* Set string terminator byte	*/

	FindString(HexBuffer,x);	  /* Search forward in the file	*/

	return (SUCCESS);
}

int RepeatSearch()
{
	int FindString(char *cbuff,unsigned strlth);

	if ( (DataChanged) )	      /* If anything changed, update it	*/
		if ( (WriteRecord()) == FAIL )
			abort();

	if ( (HexBuffer[0] != NULL) )  /* Determine Hex or ASCII search	*/
		FindString(HexBuffer,strlen(HexBuffer));
	else
		FindString(SearchString,strlen(SearchString));

	return (SUCCESS);
}

int FindString(char *cbuff,unsigned strlth)
{
	int s;
	unsigned cpos,rem;
	long rrn;
	char *gp, *buf;

	ShowChar(CurrentPos,StdAttr);	/* Remove hilite from character	*/

	cpos = CurrentPos + 1;		    /* Start with next position	*/

	for (rrn = NextRelRecNo; rrn < MaxRecords; rrn++) /* to the end	*/
		{
		if ( (ReadRecord(rrn)) == FAIL )
			return (FAIL);

		buf = InputBuffer + cpos;    /* After the initial pass,	*/
		rem = ByteCount - cpos;	  /* cpos will be reset to zero	*/

		while ( rem > 0 )	      /* Scan the entire record	*/
			{
			if ( (memicmp(buf,cbuff,strlth)) == 0 )
				break;

			buf++;	 /* Increment by one character position	*/
			rem--;
			}

		if ( (rem > 0) ) /* Allow overlap beyond current record	*/
			break;		/* Hit!	*/

		cpos = 0;	     /* Get ready to do the next record	*/
		}

	if ( (rrn == MaxRecords) )	       /* Did we find anything?	*/
		{
		ReadRecord(NextRelRecNo);			  /* No	*/
		ShowMessage("[%s] not found ...",SearchString);
		return (FAIL);
		}

	CurrentPos = buf - InputBuffer;	  /* Set the character position	*/

	if ( (rrn != NextRelRecNo) )		     /* Current record?	*/
		DisplayRecord(rrn);				  /* No	*/

	ShowMessage("[%s] found on Page %lu",SearchString,NextRelRecNo + 1);

	return (SUCCESS);
}

void ShowChar(int pos,int attr)
{
	int crow,hex_col,ascii_col;
	unsigned c;
	char addr[3];

	if ( (CurrentPos < 0) )			   /* Lower Range Check	*/
		CurrentPos = 0;

	if ( (CurrentPos > (ByteCount - 1)) )	   /* Upper Range Check	*/
		CurrentPos = ByteCount - 1;

	crow = FirstRow + (pos / 16);		   /* Determine the Row	*/
	hex_col = ((pos % 16) * 3) + 13;	/* Determine Hex Column	*/
	ascii_col = (pos % 16) + 62;	      /* Determine ASCII Column	*/

	c = *(InputBuffer + pos);		   /* Get the Character	*/

	sprintf(addr,"%02X",c);		  /* Convert to displayable hex	*/

	SetCursor(crow,hex_col);		 /* Position the Cursor	*/

	if ( (c > 127) & (attr == StdAttr) )	/* Display the hex char	*/
		WriteString(&addr,LightRed);  /* Red, if high bit is on	*/
	else
		WriteString(&addr,attr);

	if ( !(isprint(c = toascii(c))) )
		c = '.';		    /* Default for unprintables	*/

	SetCursor(crow,ascii_col);		 /* Position the Cursor	*/

	WriteCharacter(c,attr);		 /* Display the ASCII character	*/

	SetCursor(crow,hex_col);	/* Reset cursor to hex position	*/
}

/************************************************************************/
/* Edit the record into hex and ascii for display.			*/
/************************************************************************/

int DisplayRecord(long rrn)
{
	int ptr;
        unsigned c,seg,off;

	if ( (ReadRecord(rrn)) != SUCCESS )
		return (FAIL);

	NextRelRecNo = rrn;

	ClearWindow(FirstRow,1,LastRow,78);	       /* Clear the box	*/

	if ( (CurrentPos / 16) > (RowCount - 1) )	/* Short Record	*/
		CurrentPos = ((RowCount - 1) * 16) + (CurrentPos % 16);

	Cprintf(FirstRow - 2,71,Yellow,"% 7lu",NextRelRecNo + 1); /* Page */

	for (ptr = 0; ptr < ByteCount ; ptr++)	 /* Display the address	*/
		{
 		if ( (ptr % 16 == 0) )
			{
			seg = ByteOffset / 65536;
			off = (ByteOffset % 65536) + ptr;

			Cprintf(FirstRow+(ptr/16),2,LightCyan,"%Fp",MK_FP(seg,off));
			}

		ShowChar(ptr,StdAttr);	       /* Display the character	*/
		}

	ClearMessage();

	return (SUCCESS);
}

/************************************************************************/
/* Read the record designated by the Relative Record Number		*/
/************************************************************************/

int ReadRecord(long rrn)
{
	if ( (DataChanged) )	/* If anything changed, update the page	*/
		if ( (WriteRecord()) == FAIL )
			return (FAIL);

	ByteOffset = rrn * RecordSize;		    /* Offset into file	*/

	if ( (lseek(FileHandle,ByteOffset,SEEK_SET)) == -1L ) /* Target */
		{
		DosErrMsg();
		return (FAIL);
		}

	if ( (ByteCount = read(FileHandle,InputBuffer,(RecordSize * 2))) == -1 )
		{
		DosErrMsg();
		return (FAIL);
		}

	if ( (ByteCount > RecordSize) )	    /* We really read 2 records	*/
		ByteCount = RecordSize;  /* Limit the size of ByteCount	*/

	RowCount = ByteCount / 16;	/* Calculate the number of rows	*/

	if ( (ByteCount % 16 > 0) )
		RowCount++;		/* Round up for short block	*/

	return (SUCCESS);
}

/************************************************************************/
/* Write the record designated by the current Relative Record Number	*/
/************************************************************************/

int WriteRecord()
{
	int c;
	struct ffblk ffblk;

	DataChanged = FALSE;

/* Make sure this is what the user wants to do	*/

	c = GetYesNo("Page has changed.  Update this page");

/* If this is the last thing before terminating FEDIT, set the cursor	*/

	SetCursor(24,0);

	if ( (c != 'Y') )
		{
		if ( (DisplayRecord(NextRelRecNo)) == FAIL )
			return (FAIL);

		ShowMessage("Changes discarded...");
		return (SUCCESS);
		}

	if ( (lseek(FileHandle,ByteOffset,SEEK_SET)) == -1L ) /* Target	*/
		{
		DosErrMsg();
		return (FAIL);
		}

	if ( (write(FileHandle,InputBuffer,ByteCount)) == -1 )
		{
		DosErrMsg();     /* If it's READ ONLY, we're in trouble	*/
		return (FAIL);
		}

	close(FileHandle);

	if ( (FileHandle = open(InputFile,O_RDWR | O_BINARY)) == -1 )
		{
		DosErrMsg();
		return (FAIL);
		}		

	if ( (FileLength = filelength(FileHandle)) == -1L ) /* Find EOF	*/
		{
		DosErrMsg();
		return (FAIL);
		}

/* Get the new file information for the display	*/

	if ( (findfirst(InputFile,&ffblk,0)) == 0 )
		{
		FileDate = EditDate(ffblk.ff_fdate);
		FileTime = EditTime(ffblk.ff_ftime);
		}

/* Show the new last modified date and time	*/

	Cprintf(FirstRow - 2,16,Yellow,"%s",FileDate);
	Cprintf(FirstRow - 2,26,Yellow,"%s",FileTime);

	if ( (DisplayRecord(NextRelRecNo)) == FAIL )	  /* Re-display	*/
		return (FAIL);

	ShowMessage("Page updated successfully ...");

	return (SUCCESS);
}

/************************************************************************/
/* Open the input file for viewing/editing.				*/
/************************************************************************/

int OpenFile()
{
	int i;
	char str[63];
	struct ffblk ffblk;

/* Allocate enough space for two records.  This will allow overlap for	*/
/* string searches in Hex and ASCII.	*/

	InputBuffer = (char *) calloc(2,RecordSize);

/* If no file name was entered on the command line, ask for it now	*/

	if ( (strlen(InputFile)) == 0 )
		{
		printf("\nFile name or path name to edit ? ");
		str[0] = 60;
		cgets(str);

		if ( (str[2] == NULL) )
			return (EXIT);

		strcpy(InputFile,&str[2]);
		}

	strupr(InputFile);		/* Let's be neat and consistent	*/

/* Open the file with a handle and prepare for doing random reads	*/

	if ( (FileHandle = open(InputFile,O_RDWR | O_BINARY)) == -1 )
		{
		printf("\n%s\n%s\n",InputFile,sys_errlist[errno]);
		return (EXIT);
		}		

	if ( (FileLength = filelength(FileHandle)) == -1L ) /* Find EOF	*/
		{
		printf("\n%s\n%s\n",InputFile,sys_errlist[errno]);
		return (FAIL);
		}

/* Get the file information for the display	*/

	if ( (findfirst(InputFile,&ffblk,0)) == 0 )
		{
		FileDate = EditDate(ffblk.ff_fdate);
		FileTime = EditTime(ffblk.ff_ftime);
		}

	MaxRecords = FileLength / RecordSize;		/* Record Count	*/

	if ( (FileLength % RecordSize) > 0L )
		MaxRecords++;				 /* Round it Up	*/

	return (SUCCESS);
}

/************************************************************************/
/* Global Utility Functions						*/
/************************************************************************/

unsigned Convert2Hex(unsigned hex)
{
	if ( (hex > 64) )
		hex = hex - 7;			 /* Make A-F into 10-15	*/

	return (hex & 0x0f);			     /* Strip Zone bits	*/
}

void WriteCharacter(unsigned ch,int attr)        /* Write one character	*/
{
	_BH = 0;		/* Set BH to current display page	*/
	_BL = attr;		/* Set BL to the attribute		*/
	_CX = 1;		/* Set CX to repeat count		*/
	_AL = ch;		/* Character to display			*/
	_AH = 9;		/* Function code to display character	*/

	geninterrupt(0x10);	/* BIOS interrupt			*/
}

void WriteString(char *str,int attr)
{
/* Use the BIOS interrupt to display a character string	*/

	int lth,row,col;

	lth = strlen(str);

	row = Row();
	col = Col();

	_BL = attr;		/* Set BL to the attribute		*/
	_DL = col;
	_DH = row;
	_CX = lth;		/* Set CX to string length		*/
	_BH = 0;		/* Set BH to current display page	*/
	_ES = _DS;
	_AL = 1;		/* Move cursor after display		*/
	_AH = 19;		/* Function code to display string	*/
	_DI = _BP;
	_BP = str;		/* Get string pointer into es:bp	*/

	geninterrupt(0x10);	/* BIOS interrupt			*/

	_BP = _DI;		/* Restore the BP pointer		*/
}

void Cprintf(int row,int col,unsigned attr,char *format,...)
{
/* Display a formatted message at the designated row and column	*/

	char str[81];
	va_list arg;

	va_start(arg,format);
	vsprintf(str,format,arg);
	va_end(arg);

	SetCursor(row,col);

	WriteString(str,attr);
}

void ShowMessage(char *format,...)
{
/* Show a message centered in the designated message row	*/

	int col;
	char str[MAXPATH];
	va_list arg;

	va_start(arg,format);
	vsprintf(str,format,arg);
	va_end(arg);

	ClearMessage();
	SetCursor(MsgRow,40 - (strlen(str) / 2));

	WriteString(str,LightRed);
}

unsigned Wait(char *prompt)
{
/* Display a message and wait for some keyboard action	*/

	unsigned ch;

	ShowMessage(prompt);

	do {;} while ( (ch = GetKey()) == NO_KEY );

	ClearMessage();

	return (ch);
}

unsigned GetYesNo(char *prompt)
{
/* Display a message and wait for input. Make sure it's Y, N or Cancel	*/

	unsigned c;
	char msg[MAXPATH];

	strcpy(msg,prompt);
	strcat(msg," (Y/N) ?");       /* Add the (Y/N) ? to the message	*/

	do	{
		if ( (c = toupper(Wait(msg))) == CANCEL )
			break;

		} while (strchr("YN",c) == NULL);

	return (c);
}

void DrawHoriz(int row,int lcol,int rcol,unsigned ch,int attr)
{
/*	Draws a horizontal line of 'ch' with 'attr' attribute		*/
/*	starting from 'lcol' in 'row' and going until 'rcol' in 'row'.	*/

	int i;

	for ( i = lcol; i <= rcol; i++ )
		{
		SetCursor(row,i);
		WriteCharacter(ch,attr);
		}

	SetCursor(row,++rcol);
}

void DrawVert(int trow,int col,int brow,unsigned ch,int attr)
{
/*	Draws a vertical line of 'ch' with 'attr' attribute starting	*/
/*	from 'trow' in 'col' and going until 'brow' in 'col'.		*/

	int i;

	for (i = trow; i <= brow; i++)
		{
		SetCursor(i,col);
		WriteCharacter(ch,attr);
		}

	SetCursor(brow,++col);
}

void DrawBox(int trow,int lcol,int brow,int rcol,unsigned attr)
{
/*	Draws a text box on screen with the four coordinates provided	*/
/*	and with the attribute supplied.				*/

	ClearWindow(trow,lcol,brow,rcol);	  /* Clear the box area	*/

	WriteCharacter(TLCW,attr);
	DrawHoriz(trow,lcol+1,rcol-1,HORIZW,attr);
	WriteCharacter(TRCW,attr);

	DrawVert(trow+1,lcol,brow-1,VERTW,attr);
	DrawVert(trow+1,rcol,brow-1,VERTW,attr);

	SetCursor(brow,lcol);

	WriteCharacter(BLCW,attr);
	DrawHoriz(brow,lcol+1,rcol-1,HORIZW,attr);
	DrawHoriz(brow,lcol+1,rcol-1,HORIZW,attr);WriteCharacter(BRCW,attr);
}

void SetCursor(int row,int col)
{
	_DL = col;
	_DH = row;
	_AH = 2;			 /* Function code to set cursor	*/
	_BH = 0;				/* Current display page	*/

	geninterrupt(0x10);	/* BIOS interrupt			*/
}

unsigned Row()				/* Gets the current display row	*/
{
	_AH = 3;			      /* Function to get cursor	*/
	_BH = 0;			        /* Current display page	*/

	geninterrupt(0x10);	/* BIOS interrupt			*/

	return (_DH);		       /* The row is in the dh register	*/
}

unsigned Col()			     /* Gets the current display column	*/
{
	_AH = 3;			      /* Function to get cursor	*/
	_BH = 0;			        /* Current display page	*/

	geninterrupt(0x10);	/* BIOS interrupt			*/

	return (_DL);		    /* The column is in the dl register	*/
}

void ClearWindow(int trow,int lcol,int brow,int rcol)
{

/*	Scrolls a window upward 'lines', scrolling only from 'trow'	*/
/*	to 'brow'.  Sets cursor to the start of 'brow'.			*/

	int lines = brow - trow + 1;

	_BH = StdAttr;
	_CH = trow;		/* Top Row of window		*/
	_CL = lcol;		/* Left Col of window		*/
	_DH = brow;		/* Bottom Row of window		*/
	_DL = rcol;		/* Right Col of window		*/
	_AL = lines;		/* Lines to scroll		*/
	_AH = 6;		/* Scroll up Function Code	*/

	geninterrupt(0x10);	/* BIOS interrupt			*/

	SetCursor(trow,lcol);
}

void ClearMessage()
{
	ClearWindow(MsgRow,0,MsgRow,79);
}

void ClearCommand()
{
	ClearWindow(CmdRow,0,CmdRow,44);
}

static unsigned fixkey(unsigned c)
{
	if ( ((c & 0xff) == 0) )
		c = (c >> 8) + 123;

	return (c & 0xff);
}

unsigned InKey()			/* Checks for keyboard	*/
{
	if ( (bioskey(1)) == NO_KEY )
		return (NO_KEY);

	return (fixkey(bioskey(0)));
}

unsigned GetKey()	/* Wait for next keyboard character	*/
{
	unsigned c;

	do {;} while ( (c = InKey()) == NO_KEY );

	return (c);
}

char *GetString(char *prompt,int cnt)
{
/* Display a prompt and wait for string input. Look for CANCEL.	*/

	int c;
	static char str[83];

	Cprintf(CmdRow,1,Yellow,"%s",prompt);

	if ( (cnt > 80) )
		cnt = 80;	 /* Don't allow more than 80 characters	*/

	str[0] = cnt;		      /* Set the max characters allowed	*/

	do {;} while ( (c = bioskey(1)) == NO_KEY );	/* Wait for key	*/

	if ( ((c & 0xff) == CANCEL) )
		{
		bioskey(0);		   /* Clear the keyboard buffer	*/
		str[2] = CANCEL;
		}
	else
		cgets(str);    /* str[0]=max chars; str[1]=actual chars	*/

	ClearCommand();			   /* Clean up before returning	*/

	return (&str[2]);	      /* Return just the entered string	*/
}

char *EditDate(unsigned date)
 {
/* Edit the file last modified date as compressed by DOS	*/

 	static char xdate[9];

	sprintf(xdate,"%02d-%02d-%02d",(date % 512) / 32,(date % 512) % 32,(date / 512) + 80);

 	return (xdate);
 }

char *EditTime(unsigned time)
{
/* Edit the file last modified time into something readable	*/

	int hr,min;
	unsigned ampm;
	static char xtime[7];

	if ( (hr = time / 2048) > 11 )	      /* Separate out the hours	*/
		{
		ampm = 'p';			  /* Determine am or pm	*/

		if ( (hr > 12) )
			hr -= 12;
		}
	else
		ampm = 'a';

	min = (time % 2048) / 32;	    /* Separate out the minutes	*/

	sprintf(xtime,"%2d:%02d%c",hr,min,ampm);

	return (xtime);
}

void DosErrMsg()
{
	ShowMessage("[%s] %s ",InputFile,sys_errlist[errno]);
}
